from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.db.models import get_model

from generic_utils.managers import GenericModelManager


def get_model_class_by_name(name):
    app_label, model_name = name.split(".")
    model = get_model(app_label, model_name, False)
    return model


class ImagesAndUserManager(models.Manager):
    """ Useful manager for models that have AttachedImage (or subclass) field
        and 'injector=GenericIngector()' manager.        
    """
    def __init__(self, *args, **kwargs):
        try:
            image_model_class = kwargs.pop('image_model_class')
        except KeyError:
            image_model_class = 'generic_images.AttachedImage'
        self.image_model_class = get_model_class_by_name(image_model_class)
        super(ImagesAndUserManager, self).__init__(*args, **kwargs)
        
    def select_with_main_images(self, limit=None, **kwargs):
        ''' Select all objects with filters passed as kwargs.   
            For each object it's main image instance is accessible as ``object.main_image``.
            Results can be limited using ``limit`` parameter.
            Selection is performed using only 2 or 3 sql queries.            
        '''
        objects = self.get_query_set().filter(**kwargs)[:limit]
        self.image_model_class.injector.inject_to(objects,'main_image', is_main=True)
        return objects
    
    def for_user_with_main_images(self, user, limit=None):
        return self.select_with_main_images(user=user, limit=limit)
            
    def get_for_user(self, user):
        objects = self.get_query_set().filter(user=user)
        return objects
                            


class AttachedImageManager(GenericModelManager):
    ''' Manager with helpful functions for attached images
    '''
    def get_for_model(self, model):
        ''' Returns all images that are attached to given model.
            Deprecated. Use `for_model` instead.
        '''
        return self.for_model(model) 
            
    def get_main_for(self, model):
        ''' Returns main image for given model '''
        try:
            return self.for_model(model).get(is_main=True)
        except models.ObjectDoesNotExist:
            return None
            



from django import template

class InvalidParamsError(template.TemplateSyntaxError):
    ''' Custom exception class to distinguish usual TemplateSyntaxErrors 
        and validation errors for templatetags introduced by ``validate_params``
        function'''
    pass

def validate_params(bits, arguments_count, keyword_positions):
    '''
        Raises exception if passed params (`bits`) do not match signature.
        Signature is defined by `arguments_count` (acceptible number of params) and
        keyword_positions (dictionary with positions in keys and keywords in values,
        for ex. {2:'by', 4:'of', 5:'type', 7:'as'}).            
    '''    
    
    if len(bits) != arguments_count+1:
        raise InvalidTagParamsError("'%s' tag takes %d arguments" % (bits[0], arguments_count,))
    
    for pos in keyword_positions:
        value = keyword_positions[pos]
        if bits[pos] != value:
            raise InvalidTagParamsError("argument #%d to '%s' tag must be '%s'" % (pos, bits[0], value))



#coding: utf-8
from django.conf.urls.defaults import *
from django.http import Http404
from django.core.urlresolvers import reverse            
        

class PluggableSite(object):
    ''' Base class for reusable apps. 
        The approach is similar to django AdminSite.
        For usage case please check photo_albums app.
    '''
    def __init__(self, instance_name, queryset, app_name,
                 object_regex = r'\d+', lookup_field = 'pk',
                 extra_context=None, template_object_name = 'object',
                 has_edit_permission = lambda request, obj: True,
                 context_processors=None):
        self.object_regex = object_regex
        self.lookup_field = lookup_field
        self.instance_name = instance_name
        self.queryset = queryset
        self.extra_context = extra_context or {}
        self.app_name = app_name
        self.has_edit_permission = has_edit_permission
        self.template_object_name = template_object_name
        self.context_processors = context_processors
        
    def reverse(self, url, args=None, kwargs=None):
        ''' Reverse an url taking self.app_name in account '''
        return reverse("%s:%s" % (self.instance_name, url,), 
                        args=args,
                        kwargs=kwargs, 
                        current_app = self.app_name)
                
        
    def check_permissions(self, request, object):
        if not self.has_edit_permission(request, object):
            raise Http404('Not allowed')
        
        
    def get_object(self, *args, **kwargs):
        '''
        If one positioned argument is given it is used as
        a value for lookup with key=self.lookup_field ('pk'
        by default). If keyword arguments are given they are
        passed as queryset's get lookup parameters.
        '''
        if args and not kwargs:
            kwargs = {self.lookup_field: args[0]} 
            args = []           
        return self.queryset.get(*args, **kwargs)
        
        
    def get_object_or_404(self, *args, **kwargs):
        '''
        If one positioned argument is given it is used as
        a value for lookup with key=self.lookup_field ('pk'
        by default). If keyword arguments are given they are
        passed as queryset's get lookup parameters. If no object
        is found Http404 exception is raised. 
        '''
        try:
            return self.get_object(*args, **kwargs)                
        except self.queryset.model.DoesNotExist:
            raise Http404('No %s matches the given query.' % self.queryset.model._meta.object_name)
    

    def get_common_context(self, object):
        context = {self.template_object_name: object, 'current_app': self.app_name}
        if (self.extra_context):
            context.update(self.extra_context)
        return context
    
    
    def get_object_and_context(self, *args, **kwargs):
        obj = self.get_object_or_404(*args, **kwargs)
        return obj, self.get_common_context(obj)
    
                    
    def make_regex(self, url):
        return r"^(?P<object_id>%s)/%s%s$" % (self.object_regex, self.app_name, url)
    
    
    def patterns(self):
        ''' This method should return url patterns 
            (like urlpatterns variable in urls.py).
            It is helpful to construct regex with make_regex method.
            Example::
            
            return patterns('photo_albums.views',                                
                                url(
                                    self.make_regex('/'),
                                    'show_album',
                                    {'album_site': self},
                                    name = 'show_album',
                                ),
                           )
        '''
        
        raise NotImplementedError


    @property
    def urls(self):
        return self.patterns(), self.app_name, self.instance_name



from django.contrib.contenttypes.models import ContentType

def get_template_search_list(app_name, object, template_name):
    """ Returns template search list.
        Example::
        
    >>> from django.contrib.auth.models import User
    >>> user=User()
    >>> get_template_search_list('my_app', user, 'list.html')
    [u'my_app/auth/user/list.html', u'my_app/auth/list.html', 'my_app/list.html']

    """
    ctype = ContentType.objects.get_for_model(object)
    return [
        u"%s/%s/%s/%s" % (app_name, ctype.app_label, ctype.model, template_name),
        u"%s/%s/%s" % (app_name, ctype.app_label, template_name,),
        u"%s/%s" % (app_name, template_name,)
    ]

    


#coding: utf-8
from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User

from composition.base import CompositionField
from generic_images.models import AttachedImage
from generic_images.signals import image_saved, image_deleted

        
class ImageCountField(CompositionField):
    ''' Field with model's attached images count.
        Value of this field is updated automatically when 
        image is added or removed. Access to this field
        doesn't produce additional 'select count(*)' query,
        data is stored in table.    
    '''
    def __init__(self, native=None):
                        
        self.internal_init(
            native = native or models.PositiveIntegerField(default=0, editable=False),
            trigger = {
                'on': (image_saved, image_deleted,),
                'do': lambda model, image, signal: AttachedImage.objects.get_for_model(model).count(),
                'field_holder_getter': lambda image: image.content_object
            }
        )        


class UserImageCountField(CompositionField):
    """ Field that should be put into user's profile (AUTH_PROFILE_MODULE). 
        It will contain number of images that are attached to corresponding User.
    """
    def __init__(self, native=None, user_attr='user'):                        
        
        def get_field_value(model, image, signal):
            return AttachedImage.objects.get_for_model(getattr(model, user_attr)).count()
        
        self.internal_init(
            native = native or models.PositiveIntegerField(default=0, editable=False),
            trigger = {
                'on': (image_saved, image_deleted,),
                'do': get_field_value,
                'field_holder_getter': lambda image: image.content_object.get_profile(),
                'sender_model': User,
            }
        )        
        
#class ImageCountField(CompositionField):
#    def __init__(self, native=None, signal=None):
#        
#        def get_field_value(model, image, signal):
##             we need to handle situation where the field with same name exists in model
##             but it is not this ImageCountField
#            if model is None:
#                return
#            ctype = ContentType.objects.get_for_model(self._composition_meta.model)
#            model_ctype = ContentType.objects.get_for_model(model)                        
#            if ctype==model_ctype:
#                try:
#                    return AttachedImage.objects.get_for_model(model).count()
#                except AttributeError:
#                    return None
#            else:
#                return 0
#                return getattr(model, self._composition_meta.name)
#        
#        self.internal_init(
#            native = native or models.PositiveIntegerField(default=0),
#            trigger = dict(
#                on = signal or (models.signals.post_save, models.signals.post_delete),
#                sender_model = AttachedImage,
#                do = get_field_value,
#                field_holder_getter = lambda image: image.content_object
#            )
#        )
#        




from django.db import models
from django.contrib.contenttypes.models import ContentType


def _pop_data_from_kwargs(kwargs):
    ct_field = kwargs.pop('ct_field', 'content_type')
    fk_field = kwargs.pop('fk_field', 'object_id')
    return ct_field, fk_field
    

class GenericInjector(models.Manager):
    ''' Manager for selecting all generic-related objects in one (two) SQL queries.
        Selection is performed for a list of objects. Resulting data is aviable as attribute 
        of original model. Only one instance per object can be selected. Example usage: 
        select (and make acessible as user.avatar) all avatars for a list of user when 
        avatars are AttachedImage's attached to User model with is_main=True attributes.
    '''
    
    def __init__(self, *args, **kwargs):
        self.ct_field, self.fk_field = _pop_data_from_kwargs(kwargs)    
        super(GenericInjector, self).__init__(*args, **kwargs)
        
    
    def inject_to(self, objects, field_name, get_inject_object = lambda obj: obj, **kwargs):
        try:
            content_type = ContentType.objects.get_for_model(get_inject_object(objects[0]))        
        except IndexError:
            return objects
        
        #get related data
        kwargs.update({
                        self.ct_field: content_type, 
                        self.fk_field+'__in': [ get_inject_object(object).pk for object in objects ]
                     })
        data = self.get_query_set().filter(**kwargs)                
        data_dict = dict((getattr(item, self.fk_field), item) for item in list(data))
        
        # add info to original data
        for object in objects:
            pk = get_inject_object(object).pk
            if data_dict.has_key(pk):
                get_inject_object(object).__setattr__(field_name, data_dict[pk])    

    
    
class GenericModelManager(models.Manager):
    """ Manager with for_model method.  """    
    
    def __init__(self, *args, **kwargs):
        self.ct_field, self.fk_field = _pop_data_from_kwargs(kwargs)    
        super(GenericModelManager, self).__init__(*args, **kwargs)
    
    def for_model(self, model):
        ''' Returns all objects that are attached to given model '''
        content_type = ContentType.objects.get_for_model(model)
        kwargs = {
                    self.ct_field: content_type, 
                    self.fk_field: model.pk
                 }
        objects = self.get_query_set().filter(**kwargs)
        return objects
                    
                


#coding: utf-8

from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.generic import GenericForeignKey

from generic_utils.managers import GenericModelManager, GenericInjector

class GenericModelBase(models.Model):
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey()

    objects = GenericModelManager()
    injector = GenericInjector()

    class Meta:
        abstract=True
        

# untested
class TrueGenericModelBase(models.Model):
    content_type = models.ForeignKey(ContentType)
    object_id = models.TextField()
    content_object = GenericForeignKey()

    objects = GenericModelManager()
    injector = GenericInjector()

    class Meta:
        abstract=True


#coding: utf-8

from django.forms import ModelForm
from generic_images.models import AttachedImage


class AttachedImageForm(ModelForm):
    class Meta:
        model = AttachedImage
        fields = ['image', 'caption']
        




from django.test import TestCase
from django.test import Client
from django.core.urlresolvers import reverse
from django.test.testcases import urlsplit, urlunsplit
from django.conf import settings


class ViewTest(TestCase):
    '''
    TestCase for view testing
    '''
                        
    def setUp(self):
        """This method is automatically called by the Django test framework."""
        self.client = Client()

    def check_url(self, url_name, status=200, kwargs=None, current_app=None):
        """check_url a URL and require a specific status code before proceeding"""
        response = self.client.get(reverse(url_name, kwargs=kwargs, current_app=current_app))
        self.failUnlessEqual(response.status_code, status)
        return response
    
    def check_login_required(self, url_name, kwargs=None, current_app=None):
        """ Check if response is a redirect to login page (ignoring GET variables) """
        response = self.client.get(reverse(url_name, kwargs=kwargs, current_app=current_app))
        
        #remove GET variables, for example '?next=..'
        scheme, netloc, path, query, fragment = urlsplit(response['Location'])
        response['Location'] = urlunsplit(('http', 'testserver', path, None, None))
                
        self.assertRedirects(response, getattr(settings, 'LOGIN_URL', '/accounts/login/'))
        return response


from django.conf import settings

'''
Example::
    THUMBNAIL_TYPES = {
                        'photo': "421x1000",
                        'photo_small': "130x130",
                        'avatar': '191x1000',
                      }

'''
THUMB_TYPES = getattr(settings,'THUMBNAIL_TYPES', {})

def thumbnail_types(request):
    '''
    A context processor to add possible thumbnail sizes to the current Context.
    Useful for managing possible sorl.thumbnails thumbnail's sizes
    '''
    return {'thumbs': THUMB_TYPES}


#!/usr/bin/env python
from distutils.core import setup

setup(
      name='django-generic-images',
      version='0.28',
      author='Mikhail Korobov',
      author_email='kmike84@gmail.com',
      url='http://bitbucket.org/kmike/django-generic-images/',      
      
      description = 'Generic images pluggable django app',
      long_description = "This app provides image model (with useful managers, methods and fields) that can be attached to any other Django model using generic relations. ",
      license = 'MIT license',
      packages=['generic_images', 'generic_utils'],
      package_data={'generic_images': ['locale/en/LC_MESSAGES/*','locale/ru/LC_MESSAGES/*']},      
      
      requires = ['django (>=1.1)'],
      
      classifiers=(
          'Development Status :: 3 - Alpha',
          'Environment :: Plugins',
          'Framework :: Django',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: BSD License',
          'Natural Language :: English',
          'Natural Language :: Russian',
          'Programming Language :: Python',
          'Topic :: Software Development :: Libraries :: Python Modules'
        ),
)